@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2019-2023 Advanced Micro Devices, Inc.
+ * Copyright (C) 2019-2024 Advanced Micro Devices, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -39,6 +39,18 @@ typedef struct
     uint8_t data;
 } hipblaslt_bf8_fnuz;
 
+#ifdef ROCM_USE_FLOAT8
+typedef struct
+{
+    uint8_t data;
+} hipblaslt_f8;
+
+typedef struct
+{
+    uint8_t data;
+} hipblaslt_bf8;
+#endif
+
 #else // __cplusplus < 201103L || (!defined(__HCC__) && !defined(__HIPCC__))
 
 #define HIP_HOST_DEVICE __host__ __device__
@@ -76,7 +88,7 @@ struct HIPBLASLT_EXPORT hipblaslt_f8_fnuz
     // default constructor
     HIP_HOST_DEVICE hipblaslt_f8_fnuz() = default;
 
-#if defined(__gfx940__)
+#if defined(__gfx940__) || defined(__gfx941__) || defined(__gfx942__)
     // device specific optimized F8 down-conversion code
 
     template <bool stochastic_rounding = false>
@@ -116,7 +128,7 @@ struct HIPBLASLT_EXPORT hipblaslt_f8_fnuz
 #endif // __gfx940__
 
     // constructor from float
-#if defined(__gfx940__)
+#if defined(__gfx940__) || defined(__gfx941__) || defined(__gfx942__)
 
     // NOTE: ON-DEVICE... always optimal bias
     explicit HIP_DEVICE hipblaslt_f8_fnuz(float                          v,
@@ -187,7 +199,7 @@ struct HIPBLASLT_EXPORT hipblaslt_f8_fnuz
     }
 
     // convert to float
-#if defined(__gfx940__)
+#if defined(__gfx940__) || defined(__gfx941__) || defined(__gfx942__)
     // upcast using device specific intrinsic
     explicit inline HIP_DEVICE operator float() const
     {
@@ -258,7 +270,7 @@ struct HIPBLASLT_EXPORT hipblaslt_bf8_fnuz
     // default constructor
     HIP_HOST_DEVICE hipblaslt_bf8_fnuz() = default;
 
-#if defined(__gfx940__)
+#if defined(__gfx940__) || defined(__gfx941__) || defined(__gfx942__)
     // device specific optimized F8 down-conversion code
 
     template <bool stochastic_rounding = false>
@@ -298,7 +310,7 @@ struct HIPBLASLT_EXPORT hipblaslt_bf8_fnuz
 #endif // __gfx940__
 
     // constructor from float
-#if defined(__gfx940__)
+#if defined(__gfx940__) || defined(__gfx941__) || defined(__gfx942__)
 
     // NOTE: ON-DEVICE... always optimal bias
     explicit HIP_DEVICE hipblaslt_bf8_fnuz(float                          v,
@@ -369,7 +381,7 @@ struct HIPBLASLT_EXPORT hipblaslt_bf8_fnuz
     }
 
     // convert to float
-#if defined(__gfx940__)
+#if defined(__gfx940__) || defined(__gfx941__) || defined(__gfx942__)
     // upcast using device specific intrinsic
     explicit inline HIP_DEVICE operator float() const
     {
@@ -428,6 +440,270 @@ struct HIPBLASLT_EXPORT hipblaslt_bf8_fnuz
     }
 };
 
+#ifdef ROCM_USE_FLOAT8
+struct HIPBLASLT_EXPORT hipblaslt_f8
+{
+    uint8_t data;
+    enum class hipblaslt_hip_f8_rounding_mode
+    {
+        standard,
+        stochastic
+    };
+
+    // default constructor
+    HIP_HOST_DEVICE hipblaslt_f8() = default;
+
+    // constructor from float
+#if defined(__gfx1200__) || defined(__gfx1201__)
+
+    // NOTE: ON-DEVICE... always optimal bias
+    explicit HIP_DEVICE hipblaslt_f8(float                          v,
+                                     hipblaslt_hip_f8_rounding_mode rm
+                                     = hipblaslt_hip_f8_rounding_mode::standard,
+                                     uint32_t rng = 0)
+    {
+        __hip_fp8_e4m3 tmp(v);
+        data = tmp.__x;
+    }
+
+    // Host only implementation using s/w simulation
+    explicit HIP_HOST
+#else
+    // both Host and DEVICE for non-gfx940 using s/w simulation
+    explicit HIP_HOST_DEVICE
+#endif
+        hipblaslt_f8(float                          v,
+                     hipblaslt_hip_f8_rounding_mode rm  = hipblaslt_hip_f8_rounding_mode::standard,
+                     uint32_t                       rng = 0)
+    {
+#ifdef hipblaslt_F8_downcast_clipping
+        data = internal::cast_to_f8<float, false /*is_funz*/>(
+            v, 3, 4, true /*clip*/, (rm == hipblaslt_hip_f8_rounding_mode::stochastic), rng);
+#else // hipblaslt_F8_downcast_clipping
+        data = internal::cast_to_f8<float, false /*is_funz*/>(
+            v, 3, 4, false /*clip*/, (rm == hipblaslt_hip_f8_rounding_mode::stochastic), rng);
+#endif // hipblaslt_F8_downcast_clipping
+    }
+
+    // Constructor from half
+    explicit HIP_HOST_DEVICE hipblaslt_f8(_Float16                       v,
+                                          hipblaslt_hip_f8_rounding_mode rm
+                                          = hipblaslt_hip_f8_rounding_mode::standard,
+                                          uint32_t rng = 0)
+        : hipblaslt_f8((float)v, rm, rng)
+    {
+    }
+    // constructor from bfloat16
+    explicit HIP_HOST_DEVICE hipblaslt_f8(hip_bfloat16                   v,
+                                          hipblaslt_hip_f8_rounding_mode rm
+                                          = hipblaslt_hip_f8_rounding_mode::standard,
+                                          uint32_t rng = 0)
+        : hipblaslt_f8((float)v, rm, rng)
+    {
+    }
+    // constructor from int
+    explicit HIP_HOST_DEVICE hipblaslt_f8(int                            v,
+                                          hipblaslt_hip_f8_rounding_mode rm
+                                          = hipblaslt_hip_f8_rounding_mode::standard,
+                                          uint32_t rng = 0)
+        : hipblaslt_f8((float)v, rm, rng)
+    {
+    }
+    // constructor from double
+    explicit HIP_HOST_DEVICE hipblaslt_f8(double                         v,
+                                          hipblaslt_hip_f8_rounding_mode rm
+                                          = hipblaslt_hip_f8_rounding_mode::standard,
+                                          uint32_t rng = 0)
+        : hipblaslt_f8((float)v, rm, rng)
+    {
+    }
+
+    // convert to float
+#if defined(__gfx1200__) || defined(__gfx1201__)
+    // upcast using device specific intrinsic
+    explicit inline HIP_DEVICE operator float() const
+    {
+        return internal::cast_to_f32_from_f8(data, __HIP_E4M3);
+    }
+
+    explicit inline HIP_HOST operator float() const
+#else // non gfx1200
+    explicit inline HIP_HOST_DEVICE operator float() const
+#endif
+    {
+        return internal::cast_from_f8<float, false /*is_funz*/>(data, 3, 4, false);
+    }
+
+    // convert to half
+    explicit inline HIP_HOST_DEVICE operator _Float16() const
+    {
+        return _Float16(float(*this)); // convert to float, then convert to f16
+    }
+
+    // convert to bfloat16
+    explicit inline HIP_HOST_DEVICE operator hip_bfloat16() const
+    {
+        return hip_bfloat16(float(*this)); // convert to float, then convert to f16
+    }
+
+    // check for zero
+    inline HIP_HOST_DEVICE bool is_zero() const
+    {
+        return data == 0x00;
+    }
+
+    // check for nan
+    inline HIP_HOST_DEVICE bool is_nan() const
+    {
+        return (data & 0x7f) == 0x7f;
+    }
+
+    // check for inf
+    inline HIP_HOST_DEVICE bool is_inf() const
+    {
+        return false;
+    }
+
+    // assignment overloading only from the same F8 types
+    inline __host__ __device__ hipblaslt_f8& operator=(const hipblaslt_f8& a)
+    {
+        data = a.data;
+        return *this;
+    }
+};
+
+struct HIPBLASLT_EXPORT hipblaslt_bf8
+{
+    uint8_t data;
+    enum class hipblaslt_hip_f8_rounding_mode
+    {
+        standard,
+        stochastic
+    };
+
+    // default constructor
+    HIP_HOST_DEVICE hipblaslt_bf8() = default;
+
+    // constructor from float
+#if defined(__gfx1200__) || defined(__gfx1201__)
+
+    // NOTE: ON-DEVICE... always optimal bias
+    explicit HIP_DEVICE hipblaslt_bf8(float                          v,
+                                      hipblaslt_hip_f8_rounding_mode rm
+                                      = hipblaslt_hip_f8_rounding_mode::standard,
+                                      uint32_t rng = 0)
+    {
+        __hip_fp8_e5m2 tmp(v);
+        data = tmp.__x;
+    }
+
+    // Host only implementation using s/w simulation
+    explicit HIP_HOST
+#else
+    // both Host and DEVICE for non-gfx940 using s/w simulation
+    explicit HIP_HOST_DEVICE
+#endif
+        hipblaslt_bf8(float                          v,
+                      hipblaslt_hip_f8_rounding_mode rm  = hipblaslt_hip_f8_rounding_mode::standard,
+                      uint32_t                       rng = 0)
+    {
+#ifdef hipblaslt_F8_downcast_clipping
+        data = internal::cast_to_f8<float, false /*is_funz*/>(
+            v, 2, 5, true /*clip*/, (rm == hipblaslt_hip_f8_rounding_mode::stochastic), rng);
+#else // hipblaslt_F8_downcast_clipping
+        data = internal::cast_to_f8<float, false /*is_funz*/>(
+            v, 2, 5, false /*clip*/, (rm == hipblaslt_hip_f8_rounding_mode::stochastic), rng);
+#endif // hipblaslt_F8_downcast_clipping
+    }
+
+    // Constructor from half
+    explicit HIP_HOST_DEVICE hipblaslt_bf8(_Float16                       v,
+                                           hipblaslt_hip_f8_rounding_mode rm
+                                           = hipblaslt_hip_f8_rounding_mode::standard,
+                                           uint32_t rng = 0)
+        : hipblaslt_bf8((float)v, rm, rng)
+    {
+    }
+    // constructor from bfloat16
+    explicit HIP_HOST_DEVICE hipblaslt_bf8(hip_bfloat16                   v,
+                                           hipblaslt_hip_f8_rounding_mode rm
+                                           = hipblaslt_hip_f8_rounding_mode::standard,
+                                           uint32_t rng = 0)
+        : hipblaslt_bf8((float)v, rm, rng)
+    {
+    }
+    // constructor from int
+    explicit HIP_HOST_DEVICE hipblaslt_bf8(int                            v,
+                                           hipblaslt_hip_f8_rounding_mode rm
+                                           = hipblaslt_hip_f8_rounding_mode::standard,
+                                           uint32_t rng = 0)
+        : hipblaslt_bf8((float)v, rm, rng)
+    {
+    }
+    // constructor from double
+    explicit HIP_HOST_DEVICE hipblaslt_bf8(double                         v,
+                                           hipblaslt_hip_f8_rounding_mode rm
+                                           = hipblaslt_hip_f8_rounding_mode::standard,
+                                           uint32_t rng = 0)
+        : hipblaslt_bf8((float)v, rm, rng)
+    {
+    }
+
+    // convert to float
+#if defined(__gfx1200__) || defined(__gfx1201__)
+    // upcast using device specific intrinsic
+    explicit inline HIP_DEVICE operator float() const
+    {
+        return internal::cast_to_f32_from_f8(data, __HIP_E5M2);
+    }
+
+    explicit inline HIP_HOST operator float() const
+#else // non gfx1200
+    explicit inline HIP_HOST_DEVICE operator float() const
+#endif
+    {
+        return internal::cast_from_f8<float, false /*is_funz*/>(data, 2, 5, false);
+    }
+
+    // convert to half
+    explicit inline HIP_HOST_DEVICE operator _Float16() const
+    {
+        return _Float16(float(*this)); // convert to float, then convert to f16
+    }
+
+    // convert to bfloat16
+    explicit inline HIP_HOST_DEVICE operator hip_bfloat16() const
+    {
+        return hip_bfloat16(float(*this)); // convert to float, then convert to f16
+    }
+
+    // check for zero
+    inline HIP_HOST_DEVICE bool is_zero() const
+    {
+        return data == 0x00;
+    }
+
+    // check for nan
+    inline HIP_HOST_DEVICE bool is_nan() const
+    {
+        return (data & 0x7f) > 0x7c;
+    }
+
+    // check for inf
+    inline HIP_HOST_DEVICE bool is_inf() const
+    {
+        return (data & 0x7f) == 0x7c;
+    }
+
+    // assignment overloading only from the same F8 types
+    inline __host__ __device__ hipblaslt_bf8& operator=(const hipblaslt_bf8& a)
+    {
+        data = a.data;
+        return *this;
+    }
+};
+#endif
+
 namespace std
 {
     inline hipblaslt_f8_fnuz sin(hipblaslt_f8_fnuz a)
@@ -454,6 +730,32 @@ namespace std
     {
         return a;
     }
+#ifdef ROCM_USE_FLOAT8
+    inline hipblaslt_f8 sin(hipblaslt_f8 a)
+    {
+        return hipblaslt_f8(sinf(float(a)));
+    }
+    inline hipblaslt_f8 cos(hipblaslt_f8 a)
+    {
+        return hipblaslt_f8(cosf(float(a)));
+    }
+    inline hipblaslt_bf8 sin(hipblaslt_bf8 a)
+    {
+        return hipblaslt_bf8(sinf(float(a)));
+    }
+    inline hipblaslt_bf8 cos(hipblaslt_bf8 a)
+    {
+        return hipblaslt_bf8(cosf(float(a)));
+    }
+    __device__ __host__ constexpr hipblaslt_f8 real(const hipblaslt_f8& a)
+    {
+        return a;
+    }
+    __device__ __host__ constexpr hipblaslt_bf8 real(const hipblaslt_bf8& a)
+    {
+        return a;
+    }
+#endif
 }
 
 // Special operator overloading
@@ -609,9 +911,163 @@ inline __host__ __device__ bool operator>(hipblaslt_f8_fnuz a, hipblaslt_f8_fnuz
 {
     return static_cast<float>(a) > static_cast<float>(b);
 }
+
+#ifdef ROCM_USE_FLOAT8
+inline std::ostream& operator<<(std::ostream& os, const hipblaslt_f8& f8)
+{
+    return os << float(f8);
+}
+
+inline std::ostream& operator<<(std::ostream& os, const hipblaslt_bf8& bf8)
+{
+    return os << float(bf8);
+}
+
+// all + operator overloading with mixed types
+// mixed types, always converts to f32, does computation in f32, and returns float
+inline __host__ __device__ float operator+(const float fa, hipblaslt_f8 b)
+{
+    return (fa + float(b));
+}
+
+inline __host__ __device__ float operator+(const float fa, hipblaslt_bf8 b)
+{
+    return (fa + float(b));
+}
+
+inline __host__ __device__ float operator+(hipblaslt_f8 a, const float fb)
+{
+    return (float(a) + fb);
+}
+
+inline __host__ __device__ float operator+(hipblaslt_bf8 a, const float fb)
+{
+    return (float(a) + fb);
+}
+
+inline __host__ __device__ float operator+(hipblaslt_f8 a, hipblaslt_bf8 b)
+{
+    return (float(a) + float(b));
+}
+
+inline __host__ __device__ float operator+(hipblaslt_bf8 a, hipblaslt_f8 b)
+{
+    return (float(a) + float(b));
+}
+
+inline __host__ __device__ hipblaslt_f8 operator+(hipblaslt_f8 a, hipblaslt_f8 b)
+{
+    return hipblaslt_f8(float(a) + float(b));
+}
+
+inline __host__ __device__ hipblaslt_bf8 operator+(hipblaslt_bf8 a, hipblaslt_bf8 b)
+{
+    return hipblaslt_bf8(float(a) + float(b));
+}
+
+inline __host__ __device__ hipblaslt_f8& operator+=(hipblaslt_f8& a, hipblaslt_f8 b)
+{
+    return a = hipblaslt_f8(float(a) + float(b));
+}
+
+inline __host__ __device__ hipblaslt_bf8& operator+=(hipblaslt_bf8& a, hipblaslt_bf8 b)
+{
+    return a = hipblaslt_bf8(float(a) + float(b));
+}
+
+// overloading multiplication, always returns float,
+inline __host__ __device__ float operator*(hipblaslt_f8 a, hipblaslt_f8 b)
+{
+    return float(a) * float(b);
+}
+
+inline __host__ __device__ float operator*(float a, hipblaslt_f8 b)
+{
+    return (a * float(b));
+}
+
+inline __host__ __device__ float operator*(hipblaslt_f8 a, float b)
+{
+    return (float(a) * b);
+}
+
+inline __host__ __device__ float operator*(int32_t a, hipblaslt_f8 b)
+{
+    return ((float)a * float(b));
+}
+
+inline __host__ __device__ float operator*(double a, hipblaslt_f8 b)
+{
+    return ((float)a * float(b));
+}
+
+inline __host__ __device__ float operator*(hipblaslt_bf8 a, hipblaslt_bf8 b)
+{
+    return float(a) * float(b);
+}
+
+inline __host__ __device__ float operator*(float a, hipblaslt_bf8 b)
+{
+    return (a * float(b));
+}
+
+inline __host__ __device__ float operator*(hipblaslt_bf8 a, float b)
+{
+    return (float(a) * b);
+}
+
+inline __host__ __device__ float operator*(int32_t a, hipblaslt_bf8 b)
+{
+    return ((float)a * float(b));
+}
+
+inline __host__ __device__ float operator*(double a, hipblaslt_bf8 b)
+{
+    return ((float)a * float(b));
+}
+
+// overloading for mixed f8 and bf8 types
+inline __host__ __device__ float operator*(hipblaslt_f8 a, hipblaslt_bf8 b)
+{
+    return float(a) * float(b);
+}
+
+inline __host__ __device__ float operator*(hipblaslt_bf8 a, hipblaslt_f8 b)
+{
+    return float(a) * float(b);
+}
+
+// overloading for compare
+inline __host__ __device__ bool operator==(hipblaslt_f8 a, hipblaslt_f8 b)
+{
+    return (a.data == b.data);
+}
+inline __host__ __device__ bool operator==(hipblaslt_bf8 a, hipblaslt_bf8 b)
+{
+    return (a.data == b.data);
+}
+
+inline __host__ __device__ bool operator!=(hipblaslt_f8 a, hipblaslt_f8 b)
+{
+    return (a.data != b.data);
+}
+inline __host__ __device__ bool operator!=(hipblaslt_bf8 a, hipblaslt_bf8 b)
+{
+    return (a.data != b.data);
+}
+
+inline __host__ __device__ bool operator>=(hipblaslt_f8 a, hipblaslt_f8 b)
+{
+    return static_cast<float>(a) >= static_cast<float>(b);
+}
+inline __host__ __device__ bool operator>(hipblaslt_f8 a, hipblaslt_f8 b)
+{
+    return static_cast<float>(a) > static_cast<float>(b);
+}
+#endif
 // ================ Explicit downcasting to support different rounding (RNE, SR) ===============
 // NOTE: we going to remove all assignment operator overloading from other types and enforce
-// this explicit_downcast function to make any roudning behavior default
+// this explicit_downcast function to make any rounding behavior default
 // We have to explicitly call this function with SR flag
 /*
 template <typename T,
